#ifndef _GENOTYPE_H
#define _GENOTYPE_H

#include <iostream>
#include <vector>

using namespace std;

typedef unsigned short int allele_type;

/** A genotype.  We assume there are two alleles. */
struct Genotype {
  allele_type a;
  allele_type b;

  Genotype(allele_type _a, allele_type _b) {
    a = _a; b = _b;
  }

  /** If either allele is missing, we consider this as missing. */
  bool isMissing() {
    return (a == 0 || b == 0);
  }

  bool isHomozygous() {
    return (!isMissing() && a == b);
  }
  
  bool isHeterozygous() {
  	return (!isMissing() && a != b);
  }
  
  /** Converts this to a SNP genotype using the usual mapping. */
  string toSNP() {
  	char nucleotides[] = "NACGT";
  	string s = "NN";
  	s[0] = nucleotides[a];
  	s[1] = nucleotides[b];
  	// note that even if one allele is missing, we still print
  	// whichever allele we know
  	return s;
  }
};

#endif
