#ifndef _IBD_H
#define _IBD_H

#include <iostream>
#include <vector>

#include "Marker.h"

using namespace std;

/** Information about IBD between two people. */
struct IBD {

	/** The marker from which this IBD info was derived.  May not be used... */
	Marker *m;

	// needed by STL for some reason...
	IBD() {}

	// For this constructor, we divide the p1 probability evenly
	// between the two sides.  Hopefully this won't cause problems.
  	IBD(Marker *_m, double p0, double p1, double p2) {
		m = _m; _p0 = p0; _p1Paternal = _p1Maternal = 0.5 * p1; _p2 = p2;
	}

	// This constructor requires knowing the probabilities of all four states.
	IBD(Marker *_m,
			double p0, double p1Paternal, double p1Maternal, double p2) {
		m = _m;
		_p0 = p0;
		_p1Paternal = p1Paternal;
		_p1Maternal = p1Maternal;
		_p2 = p2;
	}

	// These are to allow flexibility in representation.
	double p0() { return _p0; }
	double p1() { return _p1Paternal + _p1Maternal; }
	double p2() { return _p2; }
	double p1Paternal() { return _p1Paternal; }
	double p1Maternal() { return _p1Maternal; }
	
	/** Equality is just based on the numbers, not which marker it was. */
	bool operator==(IBD i) {
		return (_p0 == i.p0() &&
		_p1Paternal == i.p1Paternal() && _p1Maternal == i.p1Maternal() &&
		_p2 == i.p2());
	}
	
	/** Probability of paternal sharing. */
	double paternal() {
		return _p1Paternal + _p2;
	}
	
	/** Probability of maternal sharing. */
	double maternal() {
		return _p1Maternal + _p2;
	}
	
private:
  /** Probability of sharing 0, 1, or 2 alleles, respectively.
  	Note that p1 is split between maternal and paternal sides. */
  double _p0, _p1Paternal, _p1Maternal, _p2;
};

#endif
