#ifndef _IBDTABLE_H
#define _IBDTABLE_H

#include <iostream>
#include <map>
#include <vector>

#include "IBD.h"
#include "Pedigree.h"

using namespace std;

struct IBDTable {
	// map<pair<Person *, Person *>, map<Marker *, IBD> > ibd;

	/** This wastes some space, but otherwise is reasonably straightforward. */
	map<pair<Person *, Person *>, map<string, map<int, IBD> > > ibd;

  /** Reads in IBD information. 
 */
  IBDTable(istream &is, Pedigree &p);

  /** Gets the estimated IBD for two people, at some location. */
  IBD getIBD(Person *a, Person *b, string chrom, int loc);
  
	/** Gets the sharing on one side (paternal or maternal), at
	some location.  paternalSide determines which side.
	This function is a temporary hack;
	at some point we'll probably switch to just using multipoint IBD.
	But since, for now, we're using "cases where IBD is definitely known",
	this is needed for the sib-inference case -- if the maternal side is
	known at several adjacent markers, we don't care if the paternal side
	is unknown.
		This will say "unknown" in some cases when it shouldn't, when
	only maternal, then paternal, then maternal phase is known.  Not
	worrying about this for now. */
	double getSharingOneSide
		(Person *a, Person *b, string chrom, int loc, bool paternalSide);
		
	/** This determines whether it has extended IBD information based
		* on the first line of the IBD file. */
	bool isExtendedFile() { return _isExtendedFile; }

private:
  /** isExtendedFile should be true iff the IBD file includes
		* "extended IBD states", as output by, for instance, "merlin --extended".
		* However, we can only deal with nuclear families so far. */
	bool _isExtendedFile;

	pair<Person *, Person *> sortPeople(Person *a, Person *b);
	
	/** Converts from Haldane map distance to probability of crossover.
	 * Not used yet. */
	double probOfCrossover(double cmDistance);

	/** Reads the first line, and checks that it "looks" like the
		* right kind of file (and returns true iff it looks like
		* an IBD file.)  This also sets _isExtendedFile appropriately. */
	bool readFirstLine(istream &is);

	/** Reads in one set of IBD probabilities (either extended
		* or non-extended.) */
	IBD readIBD(istream &is, Marker *m);
};
#endif
