#ifndef _PEDIGREE_H
#define _PEDIGREE_H

#include <iostream>
#include <map>
#include <vector>

#include "Marker.h"
#include "Person.h"

using namespace std;

struct Pedigree {

  /** The people in the pedigree, sorted first by family, then ID.
    May go away. */
//  vector<Person> people;

	/** People, indexed by family and person ID. */
	map<pair<string, string>, Person *> peopleByID;

	/** The markers, sorted by chromosome, then location.  May go away. */
//	vector<Marker> markers;

	/** Number of markers. */
	int numMarkers;

	/** Markers, indexed by chromosome, then location. */
	map<string, map<int, Marker *> > markersByLoc;

	/** Markers, indexed by marker name. */
	map<string, Marker *> markersByID;

  /** Reads in the data for a Pedigree from streams, which
  	should be the .ped and .map files. */
  Pedigree(istream &pedFile, istream &mapFile);

	/** Writes just the pedigree file (".ped") to a stream. */
	void writePedToStream(ostream &os);

	/** Writes just the pedigree's genotypes to a stream,
		in CSV format. */
	void writePedigreeAsCSV(ostream &os);
	
	/** Writes just the pedigree's genotypes in a "compressed" format,
		with genotypes from everyone all on one line. */
	void writePedigreeCompressed(ostream &os);
	
private:
	void readMapFile(istream &mapFile);
	void readPedFile(istream &pedFile);
	void writePerson(Person *p, ostream &os);
};

#endif
