#ifndef _SIB_GENOTYPE_INFERRER_H
#define _SIB_GENOTYPE_INFERRER_H

#include "Pedigree.h"
#include "IBDTable.h"

struct SibGenotypeInferrer {

	/** This contains the known genotype data; genotypes will be added. */
	Pedigree &ped;

	/** Stores IBD information. */
	IBDTable &ibd;

	/** Constructor. */
	SibGenotypeInferrer(Pedigree &ped, IBDTable &ibd);
	
	/** Infers some of the children's genotypes.
		These will be filled into the pedigree. */
	void inferGenotypes();

private:

	/** The person we're inferring genotypes for, and the
		parents and grandparents of that person. */
	Person *p, *sib, *father, *mother;
	
	/** Infers some of the children's genotypes.
		These will be filled into the pedigree. */
	void inferGenotypesAtAllMarkers();

	/** Infers one genotype for Person p. */
	void inferGenotype(Marker &m);
	
	/** Infers one allele. */
	short inferAllele
		(Marker &m, Person *p, Person *sib, Person *parent1, Person *parent2);

	/** Return the other allele of a pair. */
	short otherAllele(Genotype g, short allele);
};

#endif
