
#include <stdlib.h>

#include <fstream>
#include <iostream>
#include <iterator>
#include <vector>

#include "Genotype.h"
#include "GenotypeInferrer.h"
#include "IBDTable.h"
#include "Pedigree.h"
#include "SibGenotypeInferrer.h"

using namespace std;

/** Simple command-line argument parser. */
map<string, string> getArgs(int argc, char **argv) {
	map<string, string> args;

	for(int i=1; i<argc; i++) {
		if (argv[i][0] == '-') {
			if (i<argc && argv[i+1][0] != '-') {
				args[argv[i]] = argv[i+1];
				i++;
			}
			else
				args[argv[i]] = "";
		}
	}
	return args;
}

/** Attempts to open a file, with some error checking. */
void openFile
(ifstream &f, string fileName, string optionName, string optionFlag) {
	f.open(fileName.c_str());
	if (!f.is_open()) {
		cerr << "Error: couldn't open " << optionName
			<< " file \"" << fileName
			<< "\" (which should be specified with the "
			<< optionFlag << " flag)" << endl;
		exit(1);
	}
}

void printHeader() {
	cerr << "Genotype Inference program, v.0.4" << endl;
	cerr << "Copyright 2006 Josh Burdick (jburdick@gradient.cis.upenn.edu)" << endl;
	cerr << "More information about this program is available at" << endl;
	cerr << "http://sourceforge.net/projects/genoinfer/" << endl;
}

int main(int argc, char** argv) {
	map<string, string> args = getArgs(argc, argv);

	// print banner, and link to more information
	printHeader();
	
	// ??? these open in text mode, I think.  Is that what we want it to do?
	ifstream pedFile, mapFile, ibdFile;
	openFile(pedFile, args["-p"], "pedigree", "-p");
	openFile(mapFile, args["-m"], "map", "-m");
	openFile(ibdFile, args["-i"], "IBD", "-i");

	// read in pedigree and IBD file
  Pedigree p(pedFile, mapFile);
  cerr << "read in pedigree and map files" << endl;
	IBDTable ibd(ibdFile, p);
	cerr << "read in IBD table" << endl;
	
	pedFile.close();
	mapFile.close();
	ibdFile.close();

	// Do genotype inference.  For now, we only infer for sibs, or for
	// children, depending on a command-line arg.
	if (args["--infer"] == "sibs") {
		SibGenotypeInferrer g(p, ibd);
		g.inferGenotypes();
	}
	else {
		// default to doing grandparent-parent inference
		GenotypeInferrer g(p, ibd);
		g.inferGenotypes();
	}
	
	// Print the resulting pedigree file, either (by default) as a pedigree
	// file (in the same format as the previous pedigree file), or in a
	// "compressed" form, with all the genotypes for a given marker
	// in one row.	For now, writes to standard output.
	if (args["--outputFormat"] == "compact")
		p.writePedigreeCompressed(cout);
	else if (args["--outputFormat"] == "csv")
		p.writePedigreeAsCSV(cout);
	else
		// FIXME: should complain about bad option
		p.writePedToStream(cout);
  return 0;
}
